/**
 * data_new.jsx: append 'data_new' fieldset.
 *
 * @DataNew, must be capitalized in order for reactjs to render it as a
 *     component. Otherwise, the variable is rendered as a dom node.
 *
 * Note: this script implements jsx (reactjs) syntax.
 */

import SupplyDatasetFile from '../input-data/supply_dataset_file.jsx';
import SupplyDatasetUrl from '../input-data/supply_dataset_url.jsx';
import checkValidString from './../validator/valid_string.js';
import ModelType from './../model/model_type.jsx';

var DataNew = React.createClass({
  // initial 'state properties'
    getInitialState: function() {
        return {
            value_title: null,
            value_dataset_type: '--Select--',
            value_model_type: '--Select--',
            render_submit: false
        };
    },
  // update 'state properties'
    changeDatasetType: function(event){
        var datasetType = event.target.value;

        if (
            datasetType && datasetType != '--Select--' &&
            checkValidString(datasetType)
        ) {
            this.setState({value_dataset_type: event.target.value});
            this.props.onChange({render_submit: false});
        }
        else {
            this.setState({value_dataset_type: '--Select--'});
            this.props.onChange({render_submit: false});
        }
    },
    changeTitle: function(event){
        var sessionTitle = event.target.value;

        if (sessionTitle && checkValidString(sessionTitle)) {
            this.setState({value_title: sessionTitle});
        }
        else {
            this.setState({value_title: null});
            this.props.onChange({render_submit: false});
        }
    },
  // update 'state properties' from child component (i.e. 'value_model_type')
    changeModelType: function(event) {
        var modelType = event.value_model_type;

        if (
            modelType && modelType != '--Select--' &&
            checkValidString(modelType)
        ) {
            this.setState({value_model_type: modelType});
            this.props.onChange({render_submit: false});
        }
        else {
            this.setState({value_model_type: '--Select--'});
            this.props.onChange({render_submit: false});
        }
    },
  // update 'state properties' from child component
    displaySubmit: function(event) {
        if (event.submitted_proper_dataset) {
            this.props.onChange({
                render_submit: event.submitted_proper_dataset
            });
        }
        else {
            this.props.onChange({render_submit: false});
        }
    },
  // triggered when 'state properties' change
    render: function(){
        var datasetType = this.state.value_dataset_type;
        var datasetTitle = this.state.value_title;
        var modelType = this.state.value_model_type;
        var Dataset = this.getSupplyDataset(
            datasetType,
            datasetTitle,
            modelType
        );

        return(
            <fieldset className='fieldset-session-data-upload'>
                <legend>Data Upload</legend>
                <fieldset className='fieldset-dataset-type'>
                    <legend>Configurations</legend>
                    <p>Provide the <i>Session Name</i>, and dataset type</p>
                    <input
                        type='text'
                        name='session_name'
                        placeholder='Session Name'
                        onInput={this.changeTitle}
                        value={this.state.value_title}
                    />

                    <select
                        name='dataset_type'
                        autoComplete='off'
                        onChange={this.changeDatasetType}
                        value={this.state.value_dataset_type}
                    >

                        <option value='' defaultValue>--Select--</option>
                        <option value='file_upload'>Upload file</option>
                        <option value='dataset_url'>Dataset URL</option>

                    </select>

                    <ModelType onChange={this.changeModelType} />
                </fieldset>

                <Dataset onChange={this.displaySubmit} />
            </fieldset>
        );
    },
  // call back: used for the above 'render' (return 'span' if undefined)
    getSupplyDataset: function(datasetType, title, modelType) {

        if (
            title && checkValidString(title) && datasetType && 
            checkValidString(datasetType) && datasetType != '--Select--' &&
            modelType && checkValidString(modelType) &&
            modelType != '--Select--'
        ) {
            return {
                file_upload: SupplyDatasetFile,
                dataset_url: SupplyDatasetUrl
            }[datasetType] || 'span';
        }
        else {
            return 'span';
        }
    }
});

// indicate which class can be exported, and instantiated via 'require'
export default DataNew
